import os.path

from configurations import values


class DirectoryPathValue(values.PathValue):
    """A PathValue requiring that its path is a directory, optionally creating it if necessary."""

    def __init__(self, *args, **kwargs):
        self.ensure_exists: bool = kwargs.pop('ensure_exists', False)
        super().__init__(*args, **kwargs)

    def setup(self, name: str) -> str:
        value = super().setup(name)
        if os.path.exists(value) and not os.path.isdir(value):
            raise ValueError(f'Path {repr(value)} is not a directory.')
        if self.ensure_exists:
            os.makedirs(value, exist_ok=True)
        return value



from typing import Type

from ._base import ComposedConfiguration, ConfigMixin


class GirderUtilsMixin(ConfigMixin):
    """
    Configure girder_utils template tags.

    This requires the `django-girder-utils` package to be installed.
    """

    @staticmethod
    def mutate_configuration(configuration: Type[ComposedConfiguration]) -> None:
        configuration.INSTALLED_APPS += ['girder_utils']



import contextlib
from typing import Type
import warnings

from configurations import Configuration, values
from configurations.base import ConfigurationBase

# With the default "late_binding=False", and "environ_name" is specified or "environ=False",
# even Values from non-included classes (e.g. `AWS_DEFAULT_REGION) get immediately evaluated and
# expect env vars to be set. Also, immediately evaluated Values cannot be tweaked effectively
# in "mutate_configuration").
values.Value.late_binding = True


# Fix RemovedInDjango50Warning, until upstream django-configurations removes them
deprecated_settings = ['USE_L10N', 'USE_DEPRECATED_PYTZ']
# Since this attribute is set by the metaclass, it exists on the already-instantiated Configuration
# class. It also will be re-set on every subclass, but since those are not instantiated yet, we
# can replace the metaclass for all subclasses with a fixed one.
for deprecated_setting in deprecated_settings:
    with contextlib.suppress(AttributeError):
        delattr(Configuration, deprecated_setting)


class FixedConfigurationBase(ConfigurationBase):
    def __new__(cls, name, bases, attrs):
        obj = super().__new__(cls, name, bases, attrs)
        for deprecated_setting in deprecated_settings:
            with contextlib.suppress(AttributeError):
                delattr(obj, deprecated_setting)
        return obj


class ComposedConfiguration(Configuration, metaclass=FixedConfigurationBase):
    """
    Abstract base for composed Configuration.

    This must always be specified as a base class after Config mixins.
    """

    @classmethod
    def pre_setup(cls):
        super().pre_setup()

        # For every class in the inheritance hierarchy
        # Reverse order allows more base classes to run first
        for base_cls in reversed(cls.__mro__):
            # If the class has "mutate_configuration" as its own (non-inherited) method
            if 'mutate_configuration' in base_cls.__dict__:
                base_cls.mutate_configuration(cls)
            elif 'before_binding' in base_cls.__dict__:
                warnings.warn(
                    'In "ConfigMixin" subclasses, '
                    '"before_binding" should be renamed to "mutate_configuration"',
                    DeprecationWarning,
                )
                base_cls.before_binding(cls)


class ConfigMixin:
    """Abstract mixin for composable Config sections."""

    @staticmethod
    def mutate_configuration(configuration: Type[ComposedConfiguration]) -> None:
        """
        Mutate the configuration before values are fully bound with environment variables.

        `configuration` refers to the final Configuration class, so settings from
        other Configs in the final hierarchy may be referenced.
        """
        pass



from typing import Type, cast

from allauth.account import app_settings as allauth_settings
from django.contrib.auth import get_user_model
from django.contrib.auth.management.commands import createsuperuser as django_createsuperuser
from django.contrib.auth.models import AbstractUser
from django.core.management import BaseCommand
from django.db.models.signals import post_save

from composed_configuration._allauth_support import (
    createsuperuser as allauth_support_createsuperuser,
)
from composed_configuration._allauth_support.receiver import verify_email_address_on_user_post_save

"""
When Allauth is configured to use a User's `email` as the `username`, override the `createsuperuser`
management command to only prompt for an email address.
"""

# If using email as username
if not allauth_settings.USERNAME_REQUIRED:
    # Expose the modified command
    Command: Type[BaseCommand] = allauth_support_createsuperuser.Command
    user_model: Type[AbstractUser] = allauth_support_createsuperuser.EmailAsUsernameProxyUser

else:
    # Expose the pristine upstream version of the command
    Command = django_createsuperuser.Command
    user_model = cast(Type[AbstractUser], get_user_model())

# Always automatically verify email addresses of newly created superusers
post_save.connect(verify_email_address_on_user_post_save, sender=user_model)
